/*
 * This file is part of MiToBo, the Microscope Image Analysis Toolbox.
 *
 * Copyright (C) 2010 - 2019
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on MiToBo, visit
 *
 *    http://www.informatik.uni-halle.de/mitobo/
 *
 */

package de.mitoboOperatorDemo;

import de.unihalle.informatik.Alida.exceptions.ALDOperatorException;
import de.unihalle.informatik.Alida.exceptions.ALDProcessingDAGException;
import de.unihalle.informatik.Alida.annotations.ALDAOperator;
import de.unihalle.informatik.Alida.annotations.ALDAOperator.Level;
import de.unihalle.informatik.Alida.annotations.Parameter;
import de.unihalle.informatik.MiToBo.core.datatypes.images.*;
import de.unihalle.informatik.MiToBo.core.datatypes.images.MTBImage.MTBImageType;
import de.unihalle.informatik.MiToBo.core.datatypes.wrapper.MTBDoubleData;
import de.unihalle.informatik.MiToBo.core.operator.*;
import de.unihalle.informatik.MiToBo.segmentation.thresholds.CalcGlobalThreshOtsu;

/**
 * Demo operator to demonstrate the use of MiToBo images.
 * <p>
 * The operator offers options to duplicate the given input image and
 * to apply a binary threshold to it.
 *
 * @author moeller
 */

// This annotation is necessary for MiToBo to automatically index the 
// operator and make it available for operator runner, command line
// runner and graphical editor Grappa.
@ALDAOperator(genericExecutionMode=ALDAOperator.ExecutionMode.ALL,
		level=Level.APPLICATION)
public class MiToBoDemoOperator extends MTBOperator {

	/**
	 * 8-bit input image.
	 */
	@Parameter( label= "Input Image", required = true, dataIOOrder = 0,
		direction = Parameter.Direction.IN, description = "Input image.")
	private MTBImageByte inImg = null;

	/**
	 * Flag to get a copy of the image.
	 */
	@Parameter( label= "Make copy", required = true, dataIOOrder = 1, 
		direction = Parameter.Direction.IN, 
		description = "Flag for doing a copy.")
	private boolean doCopy = true;

	/**
	 * Flag to get a binary image by Otsu thresholding.
	 */
	@Parameter( label= "Threshold image", required = true, dataIOOrder = 2, 
		direction = Parameter.Direction.IN, 
		description = "Flag for thresholding.")
	private boolean doThresholding = true;

	/**
	 * Contains copy of the input image, if option is enabled.
	 */
	@Parameter( label= "Copied image", required = true,
		direction = Parameter.Direction.OUT, description = "Copy.")
	private MTBImageByte copiedImg = null;

	/**
	 * Contains the thresholded image, if option is enabled.
	 */
	@Parameter( label= "Binary image", required = true,
		direction = Parameter.Direction.OUT, description = "Binary image.")
	private MTBImageByte binaryImg = null;

	/**
	 * Default constructor, always required even if nothing happens here!
	 *  @throws ALDOperatorException
	 */
	public MiToBoDemoOperator() throws ALDOperatorException {
		// nothing to do here
	}		

	/**
	 * This method does the actual work and is mandatory.
	 * @throws ALDOperatorException 
	 * @throws ALDProcessingDAGException 
	 */
	@Override
	protected void operate() 
			throws ALDOperatorException, ALDProcessingDAGException {
		
		// If a copy is requested, simply duplicate the image.
		if (this.doCopy) {
			// the type of the image returned by 'duplicate()' is equal to 
			// the type of the input image, but as the method is generic and 
			// declares 'MTBImage' as return type, a cast is necessary
			this.copiedImg = (MTBImageByte)this.inImg.duplicate();
			// give a nice title to the image
			this.copiedImg.setTitle("Copy of " + this.inImg.getTitle());
		}
		
		// If threshold flag is active, binarize image.
		if (this.doThresholding) {

			// initialize the Otsu operator...
			CalcGlobalThreshOtsu otsuOp = new CalcGlobalThreshOtsu(this.inImg);
			
			// ... and run it
			otsuOp.runOp();
			
			// get result data
			MTBDoubleData threshold = otsuOp.getOtsuThreshold();
			
			// extract a common double data object for convenience
			double thresh = threshold.getValue().doubleValue();
			
			// there is also an operator for binarization in MiToBo, but
			// for demonstration purposes we are doing that directly here
			this.binaryImg = (MTBImageByte)MTBImage.createMTBImage(
				this.inImg.getSizeX(),	this.inImg.getSizeY(), 
				this.inImg.getSizeZ(),	this.inImg.getSizeT(),	
				this.inImg.getSizeC(), MTBImageType.MTB_BYTE);
			
			// give a nice title to the image
			this.binaryImg.setTitle("Thresholded result of " 
					+ this.inImg.getTitle());
			
			// iterate over the image, first in y direction and then in x, 
			// because this is more efficient due to data organization in 
			// the MTBImage data type
			for (int y=0; y<this.binaryImg.getSizeY(); ++y) {
				for (int x=0; x<this.binaryImg.getSizeX(); ++x) {
					
					// read the input image value at position (x,y)
					int inValue = this.inImg.getValueInt(x, y);
					
					// binarize the image
					if (inValue < thresh) {
						this.binaryImg.putValueInt(x, y, 0); // 0 = black
					}
					else {
						this.binaryImg.putValueInt(x, y, 255); // 255 = white
					}
				}
			}
		}
	}
}
